#! /usr/bin/perl -w
#
# $Id: bz_dumpbugs.pl,v 1.4 2002/03/04 21:40:24 rich Exp $
# Copyright (C) 1999-2002 by Richard Dawe <richdawe@bigfoot.com>
#
# Script to dump all the bugs for a specified product and generate
# a nice HTML summary.
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

use DBI;
use CGI;

$SCRIPT_NAME      = $0;
$SCRIPT_NAME      =~ s!.+/([^/]+)$!$1!;
$SCRIPT_VERSION   = '1.3';
$SCRIPT_COPYRIGHT = <<EOT;
Copyright (C) 1999-2002 by <A HREF="mailto:richdawe\@bigfoot.com">Richard
Dawe</A>
EOT

$cgi = new CGI('');

# Parse arguments
if (defined($ARGV[0])) {
    $product = $ARGV[0];
}

if (!defined($product)) {
    print $cgi->start_html(-title => 'Error: No product specified');
    print "\n";
    print "Error: No product specified\n";
    &html_timestamp();
    print $cgi->end_html()."\n";
    exit 0;
}

# Find the bugs database source
@data_sources = DBI->data_sources('mysql');
$bugs_data_source = '';

foreach $data_source (@data_sources) {
    if ($data_source =~ m/:bugs$/) {
	$bugs_data_source = $data_source;
	last;
    }
}

if ($bugs_data_source eq '') {
    print $cgi->start_html(-title => 'Error: Bugzilla database not found');
    print "\n";
    print "Error: Bugzilla database not found\n";
    &html_timestamp();
    print $cgi->end_html()."\n";
    exit 0;
}

# Open up the bugs database
$dbh = DBI->connect($bugs_data_source, '', '');

# Prepare the bugs query
$bugs_sth = $dbh->prepare(qq{
    select * from bugs where product="$product"
});

# Display HTML header
print $cgi->start_html(-title => "Bugs in $product")."\n";
print $cgi->h1("Bugs in $product")."\n";

# Iterate over all bugs - build up a TOC as we go along.
$bugs_sth->execute();

$bugs_html = '';
$bugs_toc_short_html = 'Bug';
$bugs_toc_long_html = '';
$bug_counter = 0;

while($hash_ref = $bugs_sth->fetchrow_hashref()) {
    $bug_counter++;

    # TOC entries
    $bug_id = $$hash_ref{'bug_id'};

    $bugs_toc_short_html .= ' '.$cgi->a({-href => '#'.$bug_id}, "$bug_id");

    $bugs_toc_long_html .= $cgi->a({-href => '#'.$bug_id},
				   'Bug '.$bug_id.': '
				   .$$hash_ref{'short_desc'})."<BR>\n";

    # Look up the user's name for the 'Assigned To' field
    $assigned_to = $assigned_to_login_name = '';
    ($assigned_to,
     $assigned_to_login_name) = &decode_userid($$hash_ref{'assigned_to'});

    # If the login name is an e-mail address, generate a mailto link
    if (!defined($assigned_to_login_name)) {
	$assigned_to_login_name = '';
    }

    if ($assigned_to_login_name =~ m/\S+@\S+/) {
	$assigned_to_login_name
	    = '&lt;'.$cgi->a({-href => "mailto:$assigned_to_login_name"},
			     $assigned_to_login_name)
		.'&gt;';	
    }

    # Get long descripton
    $bugs_long_desc_sth = $dbh->prepare(qq{
	select * from longdescs where bug_id="$bug_id"
    });

    $bugs_long_desc_sth->execute();
    $long_desc_hash_ref = $bugs_long_desc_sth->fetchrow_hashref();
    $long_desc = '';

    while(defined($long_desc_hash_ref)) {
	($who, $who_login_name) = &decode_userid($$long_desc_hash_ref{'who'});
	$bug_when = $$long_desc_hash_ref{'bug_when'};
	$thetext  = $$long_desc_hash_ref{'thetext'};

	if (!defined($who_login_name ne '')) {
	    $who_login_name = '';
	}

	if (defined($thetext) && ($thetext ne '')) {
	    $long_desc .= $cgi->p(
				  $cgi->strong($bug_when).': '
				  .$who.' &lt;'.$who_login_name.'&gt;'
				  )."\n";
	    $long_desc .= $cgi->p($cgi->pre($thetext))."\n";
	}

	# Next!
	$long_desc_hash_ref = $bugs_long_desc_sth->fetchrow_hashref();
    }

    $bugs_long_desc_sth->finish();

    # URL link generation
    if (defined($long_desc) && ($long_desc ne '')) {
	$long_desc =~ s!(http://\S+)!<a href="$1">$1</a>!g;
	$long_desc =~ s!(ftp://\S+)!<a href="$1">$1</a>!g;
	$long_desc =~ s!(\S+@\S+)!<a href="mailto:$1">$1</a>!g;
    } else {
	$long_desc = $$hash_ref{'short_desc'};
    }

    # TODO: Escape fields?
    $bugs_html .= "<HR>\n\n";
    $bugs_html .=  $cgi->h2($cgi->a({-name => $bug_id},
				    'Bug '.$bug_id.': '
				    .$$hash_ref{'short_desc'}));
    $bugs_html .= "\n\n";

    if (defined($$hash_ref{'version'})) {
	$bugs_html .= $cgi->strong('Version: '.$$hash_ref{'version'})."<BR>\n";
    }

    $bugs_html .= $cgi->p(
		           'Created: '.$$hash_ref{'creation_ts'}."<BR>\n"
			  .'Priority: '.$$hash_ref{'priority'}."<BR>\n"
			  .'Severity: '.$$hash_ref{'bug_severity'}."<BR>\n"
			  .'Assigned To: '
			  .$assigned_to.' '.$assigned_to_login_name."<BR>\n"
			  .'Status: '.$$hash_ref{'bug_status'}."<BR>\n"
			  .'Resolution: '.$$hash_ref{'resolution'}."<BR>\n"
			  );
    $bugs_html .= "\n\n";
    $bugs_html .= $cgi->p(
			  'Component: '.$$hash_ref{'component'}."<BR>\n"
			  .'Platform: '.$$hash_ref{'rep_platform'}."<BR>\n"
			  .'Operating System: '.$$hash_ref{'op_sys'}."<BR>\n"
			  );
    $bugs_html .= "\n\n";
    $bugs_html .= $long_desc;
    $bugs_html .= "\n\n";
    $bugs_html .= $cgi->p($cgi->a({-href => '#toc'}, 'TOC'));
    $bugs_html .= "\n\n";
}

# Dump output
if ($bug_counter > 0) {
    $bugs_toc_short_html = $cgi->p({-align => 'center'},
				   $bugs_toc_short_html)."\n\n";
    $bugs_toc_short_html .= $cgi->p({-align => 'center'},
				    "$bug_counter bug(s) found")."\n\n";

    $bugs_toc_long_html = $cgi->p($bugs_toc_long_html)."\n";

    print $bugs_toc_short_html;
    print "<HR>\n\n";
    print $cgi->a({name => 'toc'}, $bugs_toc_long_html);
    print $bugs_html;
    print "<HR>\n\n";
    print $bugs_toc_short_html;
} else {
    print $cgi->p('No bugs found')."\n";
}

# Tidy up
$dbh->disconnect();

&html_timestamp();
print $cgi->end_html();

# - End of main program -

# -------------
# - timestamp -
# -------------

sub html_timestamp {    
    print "<HR>\n<EM><P>This page was generated on ".gmtime()
	  . " by $SCRIPT_NAME.</P>\n";
    print '<P>'.$SCRIPT_NAME.', version '.$SCRIPT_VERSION
	  .', '.$SCRIPT_COPYRIGHT."</P></EM>\n";
}

# -----------------
# - decode_userid -
# -----------------

# Returns a list containing (user name, login), based on the userid.

sub decode_userid {
    my ($userid) = @_;
    my (@res);

    $profiles_sth = $dbh->prepare(qq{
	select realname,login_name
	    from profiles where userid=$userid
    });
    $profiles_sth->execute();
    @res = $profiles_sth->fetchrow_array();
    $profiles_sth->finish();

    return(@res);
}
